﻿/*
    ajaxRequest class can be used to create objects that can make asynchronous requests to web servers and process the response.
    
    Create an object for ajaxRequest class and pass the appropriate parameters as below
    url (Mandatory)
        - the server URL to which an synchronous communication is to be made.
        - for GET communications, pass the full URL that includes the query strings as encoded key value pair. 
        - Timestamp will be added internally to make each request unique which avoid browser caching.
    method (Optional)
        - the method by which the server communication should happen. 
        - defaulted to GET
    postData (Optional)
        - pass the data which is to be posted to the server in case of a POST communication.
        - can be like key1=value1&key2=value format.
        - encode the data using prefereably, encodeURIComponent. The options are escape, encodeURI and encodeURIComponent 
    
    To execute the asynchronous request, call execute() function which accepts the following parameter
    statusChangeFunction (Optional)
        - pass the call back function that should get executed for each state change of the asynchronous request
    This callback function can have the following optional parameters in the below mentioned order
    parameter 1 - readyState, the readyState values (1,2,3,4) of the request
    parameter 2 - responseText, the response in Text format for the request made. This will be NULL until it becomes available.
    parameter 3 - responseXML, the response in XML format for the request made. This will be NULL until it becomes available.
    parameter 4 - responsestatus, the status of the request made. This can be any of the HTTP response status.    
    
    Sample Usage:
    <code>
    function testAjax() {
            var myAjaxRequest = new ajaxRequest("ajaxprocessor.aspx?name=john");
            myAjaxRequest.execute(processMyRequest);
            myAjaxRequest = null;
        }
        function processMyRequest(readyState, responseText) {            
            if (readyState == 4)
                document.getElementById("myDIV").innerHTML = responseText;
            else if (readyState == 1)
                document.getElementById("myDIV").innerHTML = "Loading...";            
        }       
    </code>
    
    @author Sanal <sanalksankar@gmail.com>
    @version 1.0
    
*/
function ajaxRequest(url, method, postData) {
    //validation start
    if (url == undefined) return false;
    this.method = method == undefined ? "GET" : method.toUpperCase();
    if (this.method != "GET" && this.method != "POST") return false;
    if (url == undefined || url == "") return false;
    //validation end
    this.url = url + ((url.indexOf('?') > 0) ? "&ajts" : "?ajts") + new Date().getTime();
    var mainCls = this;
    this.inProgress = false;
    this.xmlHttpObj = null;
    this.postData = postData;
    this.toString = function() { return "Ajax by Sanal"; }
    this.abort = function() {
        if (mainCls.inProgress) {
            mainCls.xmlHttpObj.abort();
            mainCls.inProgress = false;
            mainCls.xmlHttpObj = null;
        }
    }
    this.execute = function(statusChangeFunction) {
        try {
            // Firefox, Opera 8.0+, Safari
            mainCls.xmlHttpObj = new XMLHttpRequest();
        }
        catch (e) {
            // Internet Explorer
            try {
                mainCls.xmlHttpObj = new ActiveXObject("Msxml2.XMLHTTP");
            }
            catch (e) {
                try {
                    mainCls.xmlHttpObj = new ActiveXObject("Microsoft.XMLHTTP");
                }
                catch (e) {

                    return false; //No support for AJAX
                }
            }
        }

        mainCls.xmlHttpObj.onreadystatechange = function() {
            if (statusChangeFunction) {
                statusChangeFunction(mainCls.xmlHttpObj.readyState, typeof (mainCls.xmlHttpObj.responseText) == "unknown" ? null : mainCls.xmlHttpObj.responseText, typeof (mainCls.xmlHttpObj.responseXML) == "unknown" ? null : mainCls.xmlHttpObj.responseXML, mainCls.xmlHttpObj.readyState==4 ? mainCls.xmlHttpObj.status : null);
            }
            if (mainCls.xmlHttpObj.readyState == 4) {
                mainCls.inProgress = false;
                mainCls.xmlHttpObj = null;
            }
        }

        mainCls.xmlHttpObj.open(mainCls.method, mainCls.url, true);
        if (mainCls.method == "POST") {
            mainCls.xmlHttpObj.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
            mainCls.xmlHttpObj.setRequestHeader("Content-Length", mainCls.postData.length);
        }
        mainCls.inProgress = true;
        mainCls.xmlHttpObj.send(mainCls.method == "POST" ? mainCls.postData : null);
        return true;
    }
}